version = '4.4.1'

name = 'awsume'
author = 'Trek10, Inc'
author_email = 'package-management@trek10.com'
description = 'Awsume - A cli that makes using AWS IAM credentials easy'
license = 'MIT'
homepage = 'https://github.com/trek10inc/awsume'

message = 'Thank you for using AWSume! Check us out at https://trek10.com'
